@@ -19,65 +19,120 @@
 from __future__ import (division, print_function, absolute_import)
 
 import os
+import sys
 import gtk
 import glib
 import gobject
 import logging
-import sys
+
+from metarace import jsonconfig	# recurse?
 
 # 'check' python version
 assert sys.version >= '2.6', "Missing pre-requisite: python >= 2.6"
 
-VERSION = u'1.5.15'
+VERSION = u'1.6.8'
 LIB_PATH = os.path.realpath(os.path.dirname(__file__))
 UI_PATH = os.path.join(LIB_PATH, u'ui')
 DB_PATH = os.path.join(LIB_PATH, u'data')
 DATA_PATH = os.path.realpath(os.path.expanduser(
                              os.path.join(u'~', u'Documents', u'metarace')))
-SCB_LINELEN = 24	# default scoreboard line length
-SCB_LOGOFILE = os.path.join(UI_PATH, u'metarace_icon.svg')
+DEFAULTS_PATH = os.path.join(DATA_PATH, u'.default')
+LOGS_PATH = os.path.join(DATA_PATH, u'logs')
+LOGO_FILE = u'metarace_icon.svg'
+SYSCONF_FILE = u'metarace.json'
+PDF_TEMPLATE_FILE = u'pdf_template.json'
+HTML_TEMPLATE_FILE = u'html_template.json'
 
-def init():
+SCB_LINELEN = 24	# !! REMOVE
+sysconf = jsonconfig.config() # system-defaults, populated by init() method
+
+def init(withgtk=True):
     """Shared metarace program initialisation."""
     print (u"\n\
-metarace(" + VERSION + u") Copyright \u00a9 2010-2012  Nathan Fraser\n\
+metarace(" + VERSION + u") Copyright (C) 2010-2012  Nathan Fraser\n\
 This program comes with ABSOLUTELY NO WARRANTY.\n\
 This is free software, and you are welcome to redistribute it\n\
 under certain conditions.\n\n")
 
-    # prepare for threads
-    gtk.gdk.threads_init()
-    try:
-        glib.threads_init() 
-    except:
-        gobject.threads_init()
-
-    # fix the menubar accel mapping
-    mset = gtk.settings_get_default()
-    mset.set_string_property('gtk-menu-bar-accel', 'F24', 'override')
-
-    # set the global default window icon
-    try:
-        gtk.window_set_default_icon_from_file(SCB_LOGOFILE)
-    except:
-        SCB_LOGOFILE = os.path.join(UI_PATH, u'metarace_icon.png')
-        gtk.window_set_default_icon_from_file(SCB_LOGOFILE)
+    # Check for data path and change working directory
+    mk_data_path()
+    os.chdir(DATA_PATH)
 
     # Set global logging options
     logging._srcfile = None
     logging.logThreads = 0
     logging.logProcesses = 0
 
-    # Check for data path and change working directory
-    mk_data_path()
-    os.chdir(DATA_PATH)
+    # read in system configuration - errors here print to stderr, but
+    # are not necessarily fatal.
+    conffile = default_file(SYSCONF_FILE)
+    try:
+        if os.path.exists(conffile):
+            with open(conffile, 'rb') as f:
+                sysconf.read(f)
+    except Exception as e:
+        print(u'Error reading system config from '
+              + repr(conffile) + ': ' + repr(e))
+        
+    # Do GTK init unless disabled
+    if withgtk:
+        # Initialise threading in glib
+        try:
+            glib.threads_init() 
+        except:
+            gobject.threads_init()
+
+        # Initialise threading in GDK
+        gtk.gdk.threads_init()
+
+        # fix the gnome menubar accelerator mapping
+        mset = gtk.settings_get_default()
+        mset.set_string_property('gtk-menu-bar-accel', 'F24', 'override')
+
+        # Set the global default window icon
+        try:
+            gtk.window_set_default_icon_from_file(default_file(LOGO_FILE))
+        except:
+            pass # but ignore errors
+
+def mainloop():
+    """Call into the gtk main loop."""
+    gtk.gdk.threads_enter()
+    gtk.main()
+    gtk.gdk.threads_leave()
 
 def mk_data_path():
     """Create shared data path if it does not exist."""
     if not os.path.exists(DATA_PATH):
-        print (u"metarace: Creating data directory " + repr(DATA_PATH))
+        print (u'metarace: Creating data directory ' + repr(DATA_PATH))
         os.makedirs(DATA_PATH)
+    if not os.path.exists(DEFAULTS_PATH):
+        print (u'metarace: Creating system defaults in ' + repr(DEFAULTS_PATH))
+        os.makedirs(DEFAULTS_PATH)
 
+def default_file(filename=u''):
+    """Search the current setup for the named file and return a full path."""
+    # return named file in current working dir if not found
+    ret = filename
+
+    # first try filename as provided
+    if os.path.exists(filename):
+        pass
+    else:
+        # next try system default path
+        check = os.path.join(DEFAULTS_PATH, filename)
+        if os.path.exists(check):
+            ret = check
+        else:
+            # last check system installation
+            check = os.path.join(DB_PATH, filename)
+            if os.path.exists(check):
+                ret = check
+    # it may not be an error if the file does not exist, so no error
+    # raised at this point. A failure to open the returned file should
+    # be handled appropriately in the calling code.
+    return ret
+    
 def help_docs(window):
     """Shell out to display help documents."""
     # TODO
