@@ -29,7 +29,7 @@ first 4 characters are checked.
 		evid id is the reference number. All links generated
 		will be to the reference number instead of the evid. NOTE:
 		this normally implies resu == False, since the result will
-		be exported in the referenced event.
+		be exported in the referenced event, but this is not required.
 
 	pref	"Prefix"
 		Event prefix string
@@ -60,7 +60,7 @@ first 4 characters are checked.
 		Include this event in the printed program of events
 
 	plac	"Placeholder Count"
-		Ibclude placeholders for this many riders
+		Include placeholders for this many riders
 
 	sess	"Session"
 		Event runs in this session.
@@ -706,34 +706,3 @@ class eventdb(object):
         else:
             self.racetypes = defracetypes
 
-if __name__ == "__main__":
-    import sys
-    import pygtk
-    pygtk.require('2.0')
-
-    logging.basicConfig(level=logging.DEBUG)
-    
-    def ucb(evts, data):
-        logging.info(repr(data) + u' callback: ' + repr(evts))
-        return False
-
-    edb = eventdb()
-
-    # Check cmd line
-    filename = 'events.csv'
-    if len(sys.argv) == 2:
-        filename = sys.argv[1]
-    print(u"loading from " + repr(filename))
-    edb.load(filename)
-
-    win = gtk.Window()
-    win.set_title(u'metarace :: Event DB test')
-    win.add(uiutil.vscroller(edb.mkview(win)))
-    edb.set_startlist_cb(ucb, u'Startlist')
-    edb.set_result_cb(ucb, u'Result')
-    edb.set_program_cb(ucb, u'Program')
-  
-    win.connect('destroy', lambda *x: gtk.main_quit())
-    win.show()
-
-    gtk.main()
