@@ -1,4 +1,6 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -486,17 +488,18 @@ class f200(object):
             ret.append(rec)
         return ret
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
         cnt = 0
         sec = printing.dual_ittt_startlist()
         sec.set_single()	# 200s are one-up
 
-        sec.heading = u' '.join([
-                      u'Event', self.evno, u':',
-                      self.event[u'pref'], self.event[u'info'],
-                      u'- Start List'])
+        headvec = [u'Event', self.evno, u':', self.event[u'pref'],
+                         self.event[u'info']]
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
