@@ -35,8 +35,8 @@ import metarace
 
 from metarace import jsonconfig
 from metarace import tod
-from metarace import uscbsrv
-#from metarace import unt4
+from metarace import telegraph
+from metarace import unt4
 from metarace import strops
 from metarace import loghandler
 from metarace import uiutil
@@ -51,7 +51,7 @@ HL970_BAUD = 9600
 DISPLAYW = 192
 DISPLAYH = 16
 SCROLLMAX = 190	# chop a little less than 200 char scroll max
-SCROLLCUT = 150	# try to find a word boundary up to 150 chars
+SCROLLCUT = 160	# try to find a word boundary up to 150 chars
 EOT=0x04
 STX=0x02
 ETX=0x03
@@ -127,7 +127,7 @@ class hl970:
         """Request a re-connect to the uSCBsrv IRC connection."""
         if self.uscbport:
             self.log.info(u'Requesting re-connect to announcer: ' + 
-                           unicode(self.uscbport) + unicode(self.uscbchan))
+                           repr(self.uscbport) + repr(self.uscbchan))
         else:
             self.log.info(u'Announcer not configured.')
             # but still call into uscbsrv...
@@ -148,7 +148,6 @@ class hl970:
             self.brightness = nv
         else:
             self.brightness = 0x31
-        #self.???.set_value(self.brightness)
             
     ## Help menu callbacks
     def menu_help_docs_cb(self, menuitem, data=None):
@@ -177,7 +176,7 @@ class hl970:
                 self.log.debug(u'Timeout called early: ' + ntime.rawtime())
                 # no need to advance, desired timeout not yet reached
         except Exception as e:
-            self.log.error(u'Timeout: ' + unicode(e))
+            self.log.error(u'Timeout: ' + repr(e))
 
         # 3: Re-Schedule
         tt = tod.tod(u'now')+tod.tod(u'0.01')
@@ -196,7 +195,7 @@ class hl970:
     def process_timeout(self):
         """Perform required timeout activities."""
         # Check run state variables iff remote on:
-        if not self.remote_enable:
+        if self.remote_enable:
             # bottom line process
             if self.elapstart is not None:
                 estr = u''
@@ -221,10 +220,32 @@ class hl970:
                 estr = estr.rjust(7)
                 dstr = dstr.rjust(16-len(estr))
                 self.set_bot_line(estr + dstr)
+            elif self.ttno:	# Time trial override
+                rstr = u''
+                if self.ttrank:
+                    rstr = u'(' + self.ttrank + u')' + ' '
+                rstr = rstr.ljust(6)
+                tstr = u''
+                if self.tttime:
+                    tstr = self.tttime
+                    if u'.' not in tstr:
+                        tstr += '   '
+                tstr = tstr.rjust(10)
+                self.set_bot_line(rstr + tstr)
             elif self.timeofday:
                 self.set_bot_line(self.tod.meridian().rjust(10).center(16))
             topline = u''
-            if self.finstr:	# finish string overwrites bottom line
+            if self.ttno:
+                nostr = u''
+                if self.ttno:
+                    nostr = self.ttno
+                nostr = nostr.rjust(3)
+                nmstr = u''
+                if self.ttname:
+                    nmstr = self.ttname
+                nmstr = nmstr.ljust(12)
+                topline = nostr + u' ' + nmstr
+            elif self.finstr:	# finish string overwrites top line
                 topline = strops.truncpad(self.finstr, 16, align='c')
             elif self.laplbl or self.laptype:
                 lstr = u''	# lap prompt eg: Lap 1/23
@@ -242,7 +263,8 @@ class hl970:
                         # shorten lap time by one half interval
                         if laptm < self.maxlaptime:
                             tstr = laptm.rawtime(0)
-                if self.laptype:                # but overwrite if type
+                curtmmod = int(self.tod.timeval/3)%2
+                if self.laptype and curtmmod:      # but overwrite if type
                     tstr = self.laptype
                 tstr = u' ' + tstr.strip()
                 lstr = strops.truncpad(lstr, 16 - len(tstr))
@@ -266,7 +288,8 @@ class hl970:
 
     def app_destroy_cb(self, window, msg=u''):
         """Handle destroy signal and exit application."""
-        self.saveconfig()
+        if self.dosave:
+            self.saveconfig()
         self.log.removeHandler(self.sh)
         self.window.hide()
         self.log.info(u'App destroyed. ' + msg)
@@ -310,17 +333,17 @@ class hl970:
             h = pb.get_height()
             if w != DISPLAYW or h != DISPLAYH:
                 self.log.info(u'Source dimension: '
-                               + unicode(w) + u'x' + unicode(h)
+                               + repr(w) + u'x' + repr(h)
                                + u', scaling image.')
                 pb = pb.scale_simple(DISPLAYW, DISPLAYH, gtk.gdk.INTERP_HYPER)
             w = pb.get_width()
             h = pb.get_height()
-            self.log.debug(u'Read image: ' + unicode(w) + u'x' + unicode(h))
+            self.log.debug(u'Read image: ' + repr(w) + u'x' + repr(h))
             px = pb.get_pixels()
             pxcount = 3	# RGB
             if pb.get_has_alpha():
                 pxcount = 4	# RGBA
-            self.log.debug(u'Total pixels: ' + unicode(len(px)/pxcount))
+            self.log.debug(u'Total pixels: ' + repr(len(px)/pxcount))
             data = bytearray()
             pcount = 0
             acc = 0x00			# pixel accumulator
@@ -338,15 +361,16 @@ class hl970:
                     if pcount%8 == 0:
                         data.append(acc)
                         acc = 0x00
-            self.log.debug('Data packed mono: '
-                            + unicode(len(data)) + u' bytes')
+            self.log.debug(u'Data packed mono: '
+                            + repr(len(data)) + u' bytes')
             self.logo = data
         except Exception as e:
-            self.log.error(u'Loading logo file: ' + unicode(e))
+            self.log.error(u'Loading logo file: ' + repr(e))
 
     def but_config_clicked_cb(self, button, data=None):
         """Start config."""
         self.log.debug(u'App config cb.')
+        self.dosave = True
 
     def but_man_clicked_cb(self, button, data=None):
         """Manual display set."""
@@ -366,6 +390,7 @@ class hl970:
     
     def but_logo_clicked_cb(self, button, data=None):
         """Draw logo on display."""
+        self.set_remote_disable()
         self.log.debug(u'Draw logo.')
         self.gfxcmd(0x30, 'E', bytearray([0xf0]))
         self.gfxcmd(0x30, 'W', 
@@ -389,30 +414,37 @@ class hl970:
         if len(scrollmsg) > 0:
             self.serialwrite(chr(STX) + chr(0x80) + '11' +
                           scrollmsg.encode(ENCODING, 'replace') + chr(LF))
-            self.log.debug(u'Wrote msg: ' + repr(scrollmsg))
+            #self.log.debug(u'Wrote msg: ' + repr(scrollmsg))
         else:
-            self.log.debug('Nothing to scroll.')
+            #self.log.debug('Nothing to scroll.')
             self.scrolloff_clicked_cb(None)
 
     def scrolloff_clicked_cb(self, button, data=None):
         """Terminate a scrolling message."""
-        self.log.info(u'Scroll text terminated.')
+        #self.log.info(u'Scroll text terminated.')
         for j in [0,1,2,3]:
             self.lbuf[j] = u''	# re-display top line from obuf
         self.serialwrite(chr(STX) + chr(0x90) + '111234567890' + chr(LF))
 
+    def set_remote_enable(self):
+        self.line0.set_sensitive(False)
+        self.line1.set_sensitive(False)
+        uiutil.buttonchg(self.but_remote, uiutil.bg_armstart,'Remote')
+        self.clearlines()	# refresh cache
+        self.remote_enable = True
+
+    def set_remote_disable(self):
+        self.line0.set_sensitive(True)
+        self.line1.set_sensitive(True)
+        uiutil.buttonchg(self.but_remote, uiutil.bg_armfin,'Local')
+        self.remote_enable = False
+
     def but_remote_clicked_cb(self, button, data=None):
         """Toggle remote control."""
         if self.remote_enable:
-            self.line0.set_sensitive(False)
-            self.line1.set_sensitive(False)
-            uiutil.buttonchg(button, uiutil.bg_armstart,'Remote')
-            self.remote_enable = False
+            self.set_remote_disable()
         else:
-            self.line0.set_sensitive(True)
-            self.line1.set_sensitive(True)
-            uiutil.buttonchg(button, uiutil.bg_armfin,'Local')
-            self.remote_enable = True
+            self.set_remote_enable()
 
     def but_clear_clicked_cb(self, button, data=None):
         """Request clear on display."""
@@ -422,7 +454,7 @@ class hl970:
     ## App functions
 
     def reset(self):
-        """Reset all run state."""
+        """Reset run state."""
         self.elapstart = None
         self.elapfin = None
         self.finstr = None
@@ -432,8 +464,13 @@ class hl970:
         self.laptype = None
         self.lapstart = None
         self.lapfin = None
-        self.clearlines()	# empty local output buffer
-        self.clearall()
+        #self.ttrank = None	# tt data persists over ERP
+        #self.ttno = None
+        #self.ttname = None
+        #self.ttcat = None
+        #self.tttime = None
+        #self.clearlines()	# empty local output buffer
+        #self.clearall()
 
     def serialwrite(self, cmd):
         """Output command blocking."""
@@ -441,7 +478,7 @@ class hl970:
             if self.scb:
                 self.scb.write(cmd)
         except Exception as e:
-            self.log.error(u'Writing to scoreboard: ' + unicode(e))
+            self.log.error(u'Writing to scoreboard: ' + repr(e))
 
     def parse_reply(self, rbuf=''):
         """Try to interpret scoreboard response string."""
@@ -454,7 +491,7 @@ class hl970:
                             break
                         else:
                             di += c
-                    self.log.info(u'Ident: ' + unicode(di))
+                    self.log.info(u'Ident: ' + repr(di))
                 else:
                     self.log.info(u'Response: ' + repr(rbuf))
 
@@ -464,7 +501,7 @@ class hl970:
             if self.scb:
                 self.parse_reply(self.scb.read(1024))
         except Exception as e:
-            self.log.error(u'Reading from scoreboard: ' + unicode(e))
+            self.log.error(u'Reading from scoreboard: ' + repr(e))
 
     def gfxcmd(self, addr, code, data):
         """Send a raw graphic command."""
@@ -488,14 +525,14 @@ class hl970:
         self.serialwrite(str(cmd))
         self.serialread()
 
-    def remote_cb(self, cmd, nick):
+    def remote_cb(self, cmd, nick, chan):
         """Handle unt message from remote (in main loop)."""
         if self.remoteuser and self.remoteuser.lower() != nick.lower():
-            self.log.debug(u'Ignoring command from ' + repr(nick))
+            #self.log.debug(u'Ignoring command from ' + repr(nick))
             return False
 
-        self.log.debug(u'Command from ' + repr(nick) + u': '
-                       + repr(cmd.header) + u'::' + repr(cmd.text))
+        #self.log.debug(u'Command from ' + repr(nick) + u': '
+                       #+ repr(cmd.header) + u'::' + repr(cmd.text))
         if cmd.header == u'start':
             self.elapstart = tod.str2tod(cmd.text)
         elif cmd.header == u'finish':
@@ -527,6 +564,24 @@ class hl970:
                     self.distance = a
             except:
                 self.log.debug(u'Invalid distance: ' + repr(cmd.text))
+        elif cmd.header == u'finpanel':
+            fpvec = cmd.text.split(unichr(unt4.US))
+            self.ttrank = None
+            self.ttno = None
+            self.ttname = None
+            self.ttcat = None
+            self.tttime = None
+            if len(fpvec) > 4:	# rank/no/name/cat/time
+                if fpvec[0]:
+                    self.ttrank = fpvec[0]
+                if fpvec[1]:
+                    self.ttno = fpvec[1]
+                if fpvec[2]:
+                    self.ttname = fpvec[2]
+                if fpvec[3]:
+                    self.ttcat = fpvec[3]
+                if fpvec[4]:
+                    self.tttime = fpvec[4]
         elif cmd.header == u'scrollmsg':
             self.scrolltext.set_text(cmd.text)
             self.scrolltext.activate()
@@ -554,13 +609,14 @@ class hl970:
     def clearlines(self):
         """Empty local caches."""
         for j in range(0,8):
-            self.obuf[j] = self.lbuf[j] = u''.ljust(8)
+            self.obuf[j] = u''.ljust(8)
+            self.lbuf[j] = u'01234567'
 
     def clearall(self):
         """Call general clearing on display."""
         self.log.info(u'Clear display.')
         self.serialwrite(chr(STX) + chr(0x90) + '111234567890' + chr(LF))
-        self.serialwrite(chr(STX) + chr(0xA0) + '112345678' + chr(LF))
+        #self.serialwrite(chr(STX) + chr(0xA0) + '112345678' + chr(LF))
 
     def close_display(self):
         """Close serial port."""
@@ -572,12 +628,12 @@ class hl970:
     def reconnect_display(self):
         """Re-connect to display serial port."""
         self.close_display()
-        self.log.debug('Connecting serial port: ' + unicode(self.port))
+        self.log.debug('Connecting serial port: ' + repr(self.port))
         try:
             self.scb = serial.Serial(self.port, HL970_BAUD, timeout=0.2)
             self.stdcmd(0x40, ord('I'))
         except Exception as e:
-            self.log.error(u'Opening serial port: ' + unicode(e))
+            self.log.error(u'Opening serial port: ' + repr(e))
             self.scb = None
 
     def saveconfig(self):
@@ -586,6 +642,7 @@ class hl970:
         cw.add_section(u'hl970')
         cw.set(u'hl970', u'id', APP_ID)
         cw.set(u'hl970', u'port', self.port)
+        cw.set(u'hl970', u'brightness', unichr(self.brightness))
         cw.set(u'hl970', u'remoteport', self.remoteport)
         cw.set(u'hl970', u'remotechan', self.remotechan)
         cw.set(u'hl970', u'remoteuser', self.remoteuser)
@@ -608,20 +665,22 @@ class hl970:
         """Load app config from disk."""
         cr = jsonconfig.config({u'hl970':{
                u'id':'',
-               u'port':u'/dev/ttyUSB1',
+               u'port':u'',
                u'remoteport':u'',
-               u'remotechan':u'#announce',
-               u'remoteuser':u'uSCBsrv',
+               u'remotechan':u'',
+               u'remoteuser':u'',
                u'loglevel':unicode(logging.INFO),
                u'brightness':0x31,
                u'logo':u'',
                u'toplinetext':u'',
                u'botlinetext':u'',
                u'scrolltext':u'',
-               u'maxlaptime':u'2:00.0'
+               u'maxlaptime':u'30:00'
               }})
         cr.add_section(u'hl970')
-        cwfilename = os.path.join(self.configpath, CONFIGFILE)
+        cwfilename = metarace.default_file(CONFIGFILE)
+        # read in sysdefaults before checking for config file
+        cr.merge(metarace.sysconf, u'hl970')
 
         # re-set log file
         if self.loghandler is not None:
@@ -640,7 +699,7 @@ class hl970:
             with open(cwfilename, 'rb') as f:
                 cr.read(f)
         except Exception as e:
-            self.log.error(u'Reading app config: ' + unicode(e))
+            self.log.error(u'Reading app config: ' + repr(e))
 
         # set uSCBsrv connection
         self.remoteuser = cr.get(u'hl970', u'remoteuser')
@@ -671,7 +730,7 @@ class hl970:
         self.logofile = None
         if lgfile:
             self.logofile = lgfile
-            bmpfilename = os.path.join(self.configpath, lgfile)
+            bmpfilename = metarace.default_file(lgfile)
             self.bmp2dimg(bmpfilename)
 
         # check the maximum lap time field
@@ -705,9 +764,10 @@ class hl970:
             configpath = u'.'	# None assumes 'current dir'
         self.configpath = configpath
         self.loglevel = logging.INFO	# UI log window
+        self.dosave = False
 
         # hardware connections
-        self.remote = uscbsrv.uscbsrv()
+        self.remote = telegraph.telegraph()
         self.remoteuser = u''		# match against remote nick
         self.remoteport = u''		# only connect if requested
         self.remotechan = u'#announce'
@@ -730,11 +790,12 @@ class hl970:
         self.line1 = b.get_object('line1')
         self.scrolltext = b.get_object('scrolltext')
         self.scrolltext.grab_focus()
+        self.but_remote = b.get_object('but_remote')
         for l in [self.line0, self.line1]:
             l.modify_font(pango.FontDescription('monospace bold 24'))
         self.context = self.status.get_context_id('metarace hl970')
         b.connect_signals(self)
-        b.get_object('but_remote').clicked()
+        self.set_remote_enable()
 
         # run state
         self.running = True
@@ -755,6 +816,11 @@ class hl970:
         self.laptype = None
         self.lapstart = None
         self.lapfin = None
+        self.ttrank = None
+        self.ttno = None
+        self.ttname = None
+        self.ttcat = None
+        self.tttime = None
         self.timeofday = True  # show timeofday on bottom line?
 
         # format and connect status handlers
@@ -790,9 +856,9 @@ def main():
     app.window.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
-        app.shutdown(u'Exception from gtk.main()')
+        app.shutdown(u'Exception from main loop.')
         raise
 
 if __name__ == '__main__':
