@@ -26,12 +26,15 @@ times are reported via uscbsrv for the irtt race handler.
 
 # TODO:
 #
+#	- allow for missing gst and pygst modules on systems with no gstreamer
 #	- provide a signalling mechanism to pass messages between S/F
 #	- ui for config/startlist mgmnt
 #	- remote startlist updates
 
 import pygtk
 pygtk.require("2.0")
+
+# !! TODO: The following should be optional !!
 import pygst
 pygst.require("0.10")
 import gst
@@ -42,6 +45,7 @@ import pango
 import pangocairo
 import logging
 import os
+import sys
 
 import metarace
 from metarace import tod
@@ -60,6 +64,7 @@ LOGFILE=u'irtt_start.log'
 TIMEFONT='Nimbus Sans L Condensed Bold '
 COUNTERFONT='Nimbus Sans L Condensed Bold '
 NOHDR = [u'Start', u'start', u'time', u'Time', u'']
+AUDIOSYNCTHRESH = 50000000L	# Tolerate no more than 50ms audio desync
 
 # draw text centred
 def text_cent(cr, pr, oh, ow, msg, ch, fn):
@@ -214,8 +219,10 @@ class irtt_start(object):
                                     u'startlist':u'startlist.csv'}})
         cr.add_section(u'irtt_start')
         # check for config file
+        cfile = metarace.default_file(CONFIGFILE)
         try:
-            with open(CONFIGFILE, 'rb') as f:
+            self.log.debug(u'Reading config from: ' +repr(cfile))
+            with open(cfile, 'rb') as f:
                 cr.read(f)
         except Exception as e:
             self.log.error(u'Reading config: ' + unicode(e))
@@ -277,9 +284,12 @@ class irtt_start(object):
                 prev = r
                 if self.currider is None and r > curoft:
                     self.currider = r
+                    rvec  = self.ridermap[r]
+                    stxt = tod.tod(r).meridian()
+                    sno = rvec[1]
+                    sname = rvec[3]
                     self.log.info(u'Setting first rider to: '
-                          + u','.join([str(r), self.ridermap[r][2],
-                                        str(curoft)]))
+                          + u','.join([sno, sname]) + u' @ ' + stxt)
             # last link will be None
 
         except Exception as e:
@@ -333,6 +343,21 @@ class irtt_start(object):
                              r[1], r[3]])
         return ret
 
+    def check_play_pos(self, expect):
+        """Check and handle audio desync error."""
+        try:
+            curpos,postype = self.player.query_position(gst.FORMAT_TIME)
+            err = abs(expect - curpos)
+            if err > AUDIOSYNCTHRESH:
+                self.log.error(u'Audio desync detected: ' + repr(err) + 
+                                u'@' + repr(curpos))
+                self.player.set_state(gst.STATE_NULL)
+            else:
+                self.log.debug(u'Audio in sync, error : ' + repr(err))
+        except:
+            self.log.debug(u'Error reading audio position.')
+        return False
+
     def timeout(self, data=None):
         """Handle timeout."""
 
@@ -346,9 +371,8 @@ class irtt_start(object):
             ntod = ntime.truncate(0)
             if ntime >= self.nc.truncate(1):
                 self.tod = ntod
-                self.process_timeout()
-                # and advance one second
                 self.nc += tod.ONE
+                self.process_timeout()
             else:
                 self.log.debug(u'Timeout called early: ' + ntime.rawtime())
                 # no need to advance, desired timeout not yet reached
@@ -374,21 +398,28 @@ class irtt_start(object):
         curoft = tod2key(self.tod)
         if self.currider is not None:
             cdn = self.currider - curoft
-            if cdn == 50:	# ERROR For 30sec gaps
+            if cdn == 50 or cdn == 24:
                 self.riderstr = self.ridermap[self.currider][3]
                 self.bulb = 'red'
-                #self.remote_msg(u'LOAD: ' + self.riderlogstr(self.currider))
+                self.log.debug(u'LOAD: ' + self.riderlogstr(self.currider))
             if cdn == 15:
                 #self.player.set_state(gst.STATE_READY)
-                self.remote_msg(u'15SEC: ' + self.riderlogstr(self.currider))
+                self.player.set_state(gst.STATE_PAUSED)
                 self.riderstr = self.ridermap[self.currider][3]
                 self.bulb = 'red'
-                self.player.set_state(gst.STATE_PAUSED)
+                self.remote_msg(u'15SEC: ' + self.riderlogstr(self.currider))
             if cdn == 10:
                 self.player.set_state(gst.STATE_PLAYING)
-                #self.remote_msg(u'10SEC: ' + self.riderlogstr(self.currider))
                 self.riderstr = self.ridermap[self.currider][3]
                 self.bulb = 'red'
+                self.log.debug(u'Player started for: '
+                                 + self.riderlogstr(self.currider))
+            if cdn == 8:	# check audio stream sync
+                self.check_play_pos(2000000000L)
+            if cdn == 7:	# check audio stream sync
+                self.check_play_pos(3000000000L)
+            if cdn == 6:	# check audio stream sync
+                self.check_play_pos(4000000000L)
             if cdn == 5:
                 self.timer.armlock(True)	# always re-armlock
                 self.timer.arm(timy.CHAN_START)
@@ -397,6 +428,7 @@ class irtt_start(object):
             if cdn == 0:
                 self.bulb = 'green'
                 self.countdown = 0
+                self.log.debug(u'GO: ' + self.riderlogstr(self.currider))
             if cdn == -5:	# load sets minimum gap-> ~25sec
                 self.remote_msg(u'CLEAR: ' + self.riderlogstr(self.currider))
                 self.clear()	# also does dearm
@@ -409,13 +441,10 @@ class irtt_start(object):
                     self.countdown = cdn
                 else:
                     self.countdown = None
-            #self.log.debug(u'Countdown: ' + repr(self.countdown))
         else:
             self.riderstr = u'[finished]'
             # no more riders or error in init.
-        # Queue redraw
-        #glib.idle_add(self.draw_and_update)
-        self.draw_and_update()	# does this need to be idle added?
+        self.draw_and_update()
 
     def gst_message(self, bus, message):
         t = message.type
@@ -425,6 +454,8 @@ class irtt_start(object):
             self.player.set_state(gst.STATE_NULL)
             err, debug = message.parse_error()
             self.log.error(u"gst error: %s" % err, debug)
+        else:
+            self.log.debug(u"gst message: " + repr(message))
 
     def __init__(self):
         # logger and handler
@@ -446,12 +477,13 @@ class irtt_start(object):
 
         # Audio output
         self.player = gst.element_factory_make("playbin2", "player")
+        self.player.set_property("video-sink",
+                         gst.element_factory_make("fakesink", "fakesink"))
         bus = self.player.get_bus()
         bus.add_signal_watch()
         bus.connect("message", self.gst_message)
         self.player.set_property('uri', u'file://'
-                             + os.path.join(metarace.UI_PATH, u'start.wav'))
-        self.player.set_start_time(gst.CLOCK_TIME_NONE)
+                             + os.path.join(metarace.DB_PATH, u'start.wav'))
 
         # variables
         self.armed = False
@@ -480,13 +512,23 @@ class irtt_start(object):
 
 def main():
     """Run the application."""
+    configpath = None
+    # expand config on cmd line to realpath _before_ doing chdir
+    if len(sys.argv) > 2:
+        print(u'usage: irtt_start [configdir]\n')
+        sys.exit(1)
+    elif len(sys.argv) == 2:
+        configpath = os.path.realpath(os.path.dirname(sys.argv[1]))
+
     metarace.init()
+    if configpath is not None:
+        os.chdir(configpath)
     app = irtt_start()
     app.loadconfig()
     app.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
         app.shutdown()
         raise
