@@ -61,6 +61,29 @@ class config(object):
     def write(self, file):
         json.dump(self.__store, file, indent=2, sort_keys=True)
 
+    def dictcopy(self):
+        """Return a copy of the configuration as a dictionary object."""
+        return dict(self.__store)
+
+    def merge(self, otherconfig, section=None, key=None):
+        """Merge values from otherconfig into self."""
+        if type(otherconfig) is not config:
+            raise TypeError('Merge expects jsonconfig object.')
+        if key is not None and section is not None:	# single value import
+            if otherconfig.has_option(section, key):
+                self.set(section, key, otherconfig.get(section, key))
+        elif section is not None:
+            self.add_section(section)	# force even if not already loaded
+            if otherconfig.has_section(section):
+                for opt in otherconfig.options(section):
+                    self.set(section, opt, otherconfig.get(section, opt))
+        else:
+            for sec in otherconfig.sections():
+                if self.has_section(sec):
+                    # in this case, only add sections already defined
+                    for opt in otherconfig.options(sec):
+                        self.set(sec, opt, otherconfig.get(sec, opt))
+
     def read(self, file):
         addconf = json.load(file)
         if type(addconf) is not dict:
