@@ -1,4 +1,6 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -110,14 +112,14 @@ class omnium(object):
                            + repr(eid) + ' != ' + repr(EVENT_ID))
             self.readonly = True
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         sec = printing.section()
-        sec.heading = ' '.join([
-                      'Event', self.evno, ':',
-                  self.meet.edb.getvalue(self.event, eventdb.COL_PREFIX),
-                  self.meet.edb.getvalue(self.event, eventdb.COL_INFO),
-                      '- Start List'])
+        headvec = [u'Event', self.evno, u':', self.event[u'pref'],
+                         self.event[u'info']]
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
         sec.lines = []
         for r in self.riders:
             rno = r[COL_BIB]
