@@ -1,4 +1,6 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -524,15 +526,19 @@ class ps(object):
             auxmap.sort(self.sort_riderno)
             self.riders.reorder([a[0] for a in auxmap])
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
-        sec = printing.section()
-        sec.heading = u' '.join([
-                      u'Event', self.evno, ':',
-                        self.event[u'pref'],
-                         self.event[u'info'],
-                      u'- Start List'])
+        sec = None
+        if program: # and etype not in [u'keirin', u'sprint']:
+            sec = printing.twocol_startlist()
+        else:
+            sec = printing.section()
+        headvec = [u'Event', self.evno, u':', self.event[u'pref'],
+                         self.event[u'info']]
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
@@ -550,10 +556,11 @@ class ps(object):
             rname = strops.resname(first, last, club)
             sec.lines.append([None, rno, rname, inf, None, None])
             cnt += 1
-        if cnt > 0:
-            sec.lines.append([None, None, None])
-            sec.lines.append([None, None, 'Total riders: ' + str(cnt)])
         ret.append(sec)
+        if cnt > 0 and not program:
+            sec = printing.bullet_text()
+            sec.lines.append([None, 'Total riders: ' + str(cnt)])
+            ret.append(sec)
         return ret
 
     def do_startlist(self):
@@ -1058,6 +1065,12 @@ class ps(object):
             else:
                 self.units = 'laps'
 
+            # update series
+            ns = se.get_text()
+            if ns != self.series:
+                self.series = ns
+                self.event[u'seri'] = ns
+
             # update auto startlist spec
             nspec = as_e.get_text()
             if nspec != self.autospec:
@@ -1066,12 +1079,6 @@ class ps(object):
                     if self.autospec:
                         self.meet.autostart_riders(self, self.autospec)
 
-            # update series
-            ns = se.get_text()
-            if ns != self.series:
-                self.series = ns
-                self.meet.edb.editevent(self.event, series=ns)
-
             # xfer starters if not empty
             slist = strops.reformat_riderlist(
                           b.get_object('race_starters_entry').get_text(),
