@@ -1,4 +1,6 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -73,11 +75,12 @@ COL_INFO = 4
 COL_DNF = 5
 COL_PLACE = 6
 
-SCB_RESNAME_WIDTH = metarace.SCB_LINELEN - 11
+SCB_RESNAME_WIDTH = 24 - 11
 SCB_RESULT_FMT = [(2, u'l'), (3, u'r'), u' ',
                   (SCB_RESNAME_WIDTH, u'l'), u' ', (4, u'r')]
 
-SCB_STARTNAME_WIDTH = metarace.SCB_LINELEN - 9
+##! ! ! fix the hardcoded name formats
+SCB_STARTNAME_WIDTH = 24 - 9
 SCB_STARTERS_FMT = [(3, u'r'), u' ', (SCB_STARTNAME_WIDTH,u'l'),
                    u' ', (4,u'r')]
 
@@ -438,14 +441,23 @@ class race(object):
             self.meet.announce.setline(21, tp)
         return False
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
-        sec = printing.section()
-        sec.heading = u' '.join([
-                      u'Event', self.evno, u':', self.event[u'pref'],
-                         self.event[u'info'],
-                      u'- Start List'])
+        sec = None
+        etype = self.event[u'type']
+        if program: # and etype not in [u'keirin', u'sprint']:
+            sec = printing.twocol_startlist()
+        else:
+            sec = printing.section()
+        headvec = []
+        if etype != 'break':
+            headvec.extend([u'Event', self.evno, u':'])
+        headvec.append(self.event[u'pref'])
+        headvec.append(self.event[u'info'])
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
@@ -464,10 +476,15 @@ class race(object):
                                        r[COL_CLUB].decode('utf-8','replace'))
             sec.lines.append([None, rno, rname, inf, None, None])
             cnt += 1
-        if cnt > 0:
-            sec.lines.append([None, None, None])
-            sec.lines.append([None, None, 'Total riders: ' + str(cnt)])
+        if self.event[u'plac']:
+            while cnt < self.event[u'plac']:
+                sec.lines.append([None, None, None, None, None, None])
+                cnt += 1
         ret.append(sec)
+        if cnt > 0 and not program:
+            sec = printing.bullet_text()
+            sec.lines.append([None, 'Total riders: ' + str(cnt)])
+            ret.append(sec)
         return ret
 
     def get_startlist(self):
@@ -560,6 +577,12 @@ class race(object):
             else:
                 self.units = 'laps'
 
+            # update series
+            ns = se.get_text()
+            if ns != self.series:
+                self.series = ns
+                self.event[u'seri'] = ns
+
             # update auto startlist spec
             nspec = as_e.get_text()
             if nspec != self.autospec:
@@ -572,12 +595,6 @@ class race(object):
                     if self.evtype == 'handicap':
                         self.reorder_handicap()
 
-            # update series
-            ns = se.get_text()
-            if ns != self.series:
-                self.series = ns
-                self.event[u'seri'] = ns
-
             # xfer starters if not empty
             slist = strops.reformat_riderlist(
                           b.get_object('race_starters_entry').get_text(),
