@@ -45,6 +45,7 @@ import os
 from metarace import uiutil
 from metarace import strops
 from metarace import ucsv
+from metarace import namebank
 
 # Model column constants
 COL_BIB = 0
@@ -56,6 +57,9 @@ COL_SERIES = 5
 COL_REFID = 6
 COL_UCICODE = 7
 
+# wheeltime refid prefix
+TAGPREFIX = '058001'
+
 # Reserved rider numbers.
 RESERVED_NOS = [u'bib', u'no.', u'no', u'all', u'number', u'num', u'#']
 
@@ -87,6 +91,8 @@ class riderdb(object):
                     ir = [cell.translate(strops.PRINT_UTRANS) for cell in row]
                     if len(ir) > 1 and ir[0] != u'Refid':
                         nrefid = ir[0].strip().lower()
+                        if len(nrefid) > 6 and nrefid[0:6] == TAGPREFIX:
+                            nrefid = nrefid[6:]		# strip off prefix
                         bib = ir[1].strip().lower()
                         series = u''
                         if len(ir) > 2:
@@ -106,7 +112,7 @@ class riderdb(object):
             self.log.debug(u'Loaded ' + unicode(count) + u' refids from ' 
                             + repr(csvfile))
 
-    def load(self, csvfile=None):
+    def load(self, csvfile=None, namedb=None, overwrite=False):
         """Load riders from supplied CSV file."""
         count = 0
         if os.path.isfile(csvfile):
@@ -122,17 +128,46 @@ class riderdb(object):
                             for i in range(1,8):
                                 if len(ir) > i:
                                     nr[i] = ir[i].strip()
+
+                            # overwrite name fields if license no provided
+                            if namedb and len(ir) > 1 and ir[1].isdigit():
+                                nkey = ir[1].encode('ascii','ignore')
+                                if nkey in namedb:
+                                    # overwrite name fields with stored vals
+                                    sr = namedb[nkey]
+                                    for i in [1, 2]:	# first, last
+                                        nr[i] = sr[i]
+                                    if nr[COL_CLUB] == u'':	# club
+                                        nr[COL_CLUB] = sr[namebank.COL_CLUB]
+                                    if nr[COL_CAT] == u'':	# NRS/Origin
+                                        nr[COL_CAT] = sr[namebank.COL_STATE]
+                                    if nr[COL_REFID] == u'':	# refid
+                                        nr[COL_REFID] = sr[namebank.COL_REFID]
+                                    self.log.info(
+       u'Loaded rider detail from namebank: ' + repr(bib)+ u':' + repr(nkey))
+
                             # overwrite refid in load data if chipfile loaded
                             bibser = strops.bibser2bibstr(bib, nr[COL_SERIES])
                             if bibser in self.maptag:
                                 if nr[COL_REFID] == u'':
                                     nr[COL_REFID] = self.maptag[bibser]
-                            if self.getrider(bib, nr[COL_SERIES]) is None:
-                                self.model.append(nr)
-                                count += 1
-                            else:
-                                self.log.warn(u'Duplicate no. ignored: '
-                                              + repr(bib))
+                            oldr = self.getrider(bib, nr[COL_SERIES])
+                            if oldr is not None:
+                                # entry already exists, ignore or remove
+                                if overwrite:
+                                    self.log.info(u'Replaced: '+repr(bibser))
+                                    self.model.remove(self.model.get_iter(
+                                          oldr.get_path()))
+                                else:
+                                    self.log.warn(u'Duplicate ignored: '
+                                              + repr(bibser))
+                                    continue 
+                            # strip wheeltime prefix 
+                            if len(nr[COL_REFID]) > 8:
+                                if nr[COL_REFID][0:6] == TAGPREFIX:
+                                    nr[COL_REFID] = nr[COL_REFID][6:]
+                            self.model.append(nr)
+                            count += 1
                         else:
                             self.log.warn(u'Invalid No.: ' + repr(bib))
             self.log.debug('Loaded ' + unicode(count) + ' riders from ' 
@@ -397,7 +432,8 @@ class riderdb(object):
         for row in self.model:
             ucat = row[COL_CAT].decode('utf-8').upper()
             useries = row[COL_SERIES].decode('utf-8')
-            if ucat not in ret and (series is None or useries == series):
+            if ucat and ucat not in ret and (series is None
+                                             or useries == series):
                 ret.append(ucat)
         return ret
 
@@ -626,25 +662,3 @@ class riderdb(object):
         self.colvec = []
         self.postedit = None
         self.editwasempty = False
-
-if __name__ == "__main__":
-    import sys
-    import pygtk
-    pygtk.require('2.0')
-    
-    rdb = riderdb()
-
-    # Check cmd line
-    filename = 'riders.csv'
-    if len(sys.argv) == 2:
-        filename = sys.argv[1]
-    print("loading from " + repr(filename))
-    rdb.load(filename)
-
-    win = gtk.Window()
-    win.set_title('metarace :: Rider DB test')
-    win.add(uiutil.vscroller(rdb.mkview()))
-    win.connect('destroy', lambda *x: gtk.main_quit())
-    win.show()
-
-    gtk.main()
