@@ -602,9 +602,9 @@ def main():
     app.window.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
-        app.shutdown('Exception from gtk.main()')
+        app.shutdown('Exception from main loop.')
         raise
 
 if __name__ == '__main__':
