@@ -1,21 +1,19 @@
 
-# preliminary road race announce 'stateful' terminal
-# connects to IRC to receive UNT4 announce msg packs
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
-# Notes:
-#	- does not _yet_ do live stream negotiation
-#	- server, port, channel, nick config via road_announce.ini
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-# TODO:
-#	- properly mark private vars/methods to avoid leak to other thread
-#	- configuration dialog and test facility -> query uSCBsrv
-#	- configuration status icon for connect & status
-#	- configure map colours via ini
-#       - connect buttons to allow navigation of 'laps'
-#       - touch screen scroll/zoom in map area
-#       - touch screen scroll in tree
-#       - more natural rider search
-#	- connect riders and scale via mark on drawing area?
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import pygtk
 pygtk.require("2.0")
@@ -25,36 +23,30 @@ import glib
 import gobject
 import pango
 import threading
-import random
-import irclib
-import ConfigParser
-import time
 import os
 import sys
+import logging
 
 import metarace
 from metarace import unt4
 from metarace import tod
 from metarace import uiutil
 from metarace import strops
+from metarace import telegraph
+from metarace import jsonconfig
 
 # Global Defaults
-USCBSRV_HOST='localhost'
-USCBSRV_PORT=6667
-USCBSRV_CHANNEL='#announce'
-USCBSRV_SRVNICK='uscbsrv'
-USCBSRV_CLTNICK='roan_'+str(random.randint(100,999))
-#FONTFACE='Nimbus Sans L Bold'
 FONTFACE='Nimbus Sans L Bold Condensed'
-#FONTFACE='sans'
 TIMETICK=12	# pixels per second
 FONTSIZE=20	# font size in pixels
 MOTD=''		# Message of the day
 SCROLLDELAY=50  # Autoscroll loop delay
 SCROLLTHRESH=40 # Pause count before/after scroll
+LOGHANDLER_LEVEL=logging.DEBUG
 
 # Config filename
-CONFIGFILE='road_announce.ini'
+CONFIGFILE=u'road_announce.json'
+LOGFILE=u'road_announce.log'
 
 # Bunches colourmap
 COLOURMAP=[['#ffa0a0','red',1.0,0.1,0.1],
@@ -71,6 +63,7 @@ MAPWIDTH=STARTTIME*TIMETICK
 MAPHMARGIN=8
 MAPVMARGIN=6
 
+## TODO: replace this fn
 def roundedrecMoonlight(cr,x,y,w,h,radius_x=4,radius_y=4):
     """Draw a rounded rectangle."""
 
@@ -99,153 +92,6 @@ def roundedrecMoonlight(cr,x,y,w,h,radius_x=4,radius_y=4):
     cr.rel_curve_to (0.0, -c2, radius_x - c1, -radius_y, radius_x, -radius_y)
     cr.close_path ()
 
-class uscbio(threading.Thread):
-    def __init__(self):
-        threading.Thread.__init__(self)
-        self.running = False
-        self.cb = None
-        self.ih = irclib.IRC()
-        self.ic = self.ih.server()
-        self.np = tod.tod('now')+tod.tod('30')
-        self.hasconnnected = False
-        self.rdbuf = ''
-        self.doreconnect = False
-        self.chanstatus = False
-        self.host = USCBSRV_HOST
-        self.port = USCBSRV_PORT
-        self.channel = USCBSRV_CHANNEL
-        self.cltnick = USCBSRV_CLTNICK
-        self.srvnick = USCBSRV_SRVNICK
-        self._curpace = 0.0
-
-    def setcb(self, cb):
-        """Register the message callback function."""
-        self.cb = cb
-
-    def exit(self, msg=None):
-        """Request thread termination."""
-        self.running = False
-
-    def irc_event_cb(self, c, e):
-        """Collect and log all IRC events (debug)."""
-        print('EVENT: ' + repr(e.source()) + '/' + repr(e.eventtype()) + '/' 
-               + repr(e.target()) + ' :: ' + '/'.join(map(str, e.arguments())))
-
-    def nicknameinuse_cb(self, c, e):
-        """Handle nickname collision."""
-        self.cltnick = 'roan_'+str(random.randint(100,999))
-        self.doreconnect = True
-
-    def channel_join_cb(self, c, e):
-        """Register channel join."""
-        tg = e.target().lower()
-        if tg == self.channel:
-            self.chanstatus = True
-        
-    def channel_part_cb(self, c, e):
-        """Register channel part."""
-        tg = e.target().lower()
-        if tg == self.channel:
-            self.chanstatus = False
-
-    def unt_msg_cb(self, c, e):
-        """Handle a message packet."""
-        su = irclib.nm_to_n(e.source()).lower()
-        tg = e.target().lower()
-        if su == self.srvnick and tg == self.channel:
-            self._pacing(0.0)
-            # Have a 'broadcast' packet ... append and then search
-            self.rdbuf += unt4.decode(''.join(e.arguments()))
-            idx = self.rdbuf.find(chr(unt4.EOT))
-            if idx >= 0:
-                msgtxt = self.rdbuf[0:idx+1]
-                self.rdbuf = self.rdbuf[idx+1:]
-                if self.cb is not None:
-                    glib.idle_add(self.cb, unt4.unt4(unt4str=msgtxt))
-
-    def set_port(self, host=None, port=None, channel=None,
-                       cltnick=None, srvnick=None):
-        """Request change in irc connection."""
-        reconnect = False
-        if host is not None and host != self.host:
-            self.host = host
-            reconnect = True
-        if port is not None and port != self.port:
-            self.port = port
-            reconnect = True
-        if cltnick is not None and cltnick != self.port:
-            self.cltnick = cltnick
-            reconnect = True        
-        if channel is not None and channel != self.channel:
-            self.channel = channel
-            reconnect = True
-        if srvnick is not None:
-            self.srvnick = srvnick.lower()
-        if reconnect:
-            self.doreconnect = True
-
-    def _reconnect(self):
-        self.ic.connect(self.host, self.port, self.cltnick)
-
-    def in_channel(self):
-        return self.chanstatus
-
-    def _rejoin(self):
-        self.ic.join(self.channel)
-
-    def _pacing(self, delay=None):
-        """Adjust internal pacing delay."""
-        if delay is None:
-            self._curpace += 0.1
-            if self._curpace > 1.5:
-                self._curpace = 1.5
-        else:
-            self._curpace = delay
-        return self._curpace
-
-    def run(self):
-        """Called via threading.Thread.start()."""
-        self.running = True
-        self.ic.add_global_handler('pubmsg', self.unt_msg_cb, -10)
-        self.ic.add_global_handler('pubnotice', self.unt_msg_cb, -10)
-        self.ic.add_global_handler('join', self.channel_join_cb, -10)
-        self.ic.add_global_handler('part', self.channel_part_cb, -10)
-        self.ic.add_global_handler('kick', self.channel_part_cb, -10)
-        self.ic.add_global_handler('nicknameinuse', self.nicknameinuse_cb, -10)
-        #self.ic.add_global_handler('all_events', self.irc_event_cb, 0)
-        while self.running:
-            try:
-                self.ih.process_once(0)
-                if not self.ic.is_connected() or self.doreconnect:
-                    self.doreconnect = False
-                    self.chanstatus = False
-                    time.sleep(2)
-                    self._reconnect()
-                    # skip over preliminary connect chatter ready to 
-                    # poll channel status - avoids double channel join
-                    if self.hasconnnected:
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(0)
-                        self.ih.process_once(1)
-                    else:
-                        self.hasconnnected = True
-                if not self.in_channel():
-                    time.sleep(2)
-                    self._rejoin()
-                now = tod.tod('now')
-                if now > self.np:
-                    self.ic.ctcp('PING', self.cltnick, str(int(time.time())))
-                    self.np = now + tod.tod('30')
-                time.sleep(self._pacing())
-            except Exception as e:
-                # TODO : FIX HERE?
-                print ('Exception from uscbio: ' + str(e))
-
 class rr_announce(object):
     """Road race announcer application."""
  
@@ -253,26 +99,39 @@ class rr_announce(object):
         self.window.show()
 
     def hide(self):
-        self.window.show()
+        self.window.hide()
 
     def start(self):
         """Start io thread."""
         if not self.started:
             self.io.start()
             self.started = True
+            self.redraw_flag = True
 
     def shutdown(self):
         """Cleanly shutdown."""
-        self.io.exit()
-        self.io.join()
         self.started = False
+        self.hide()
+        self.io.exit()
+        print(u'Waiting for remote to exit...')
+        self.io.join()
+
+    def app_destroy_handler(self):
+        if self.started:
+            #self.saveconfig()
+            self.shutdown()     # threads are joined in shutdown
+        # close event and remove main log handler
+        if self.loghandler is not None:
+            self.log.removeHandler(self.loghandler)
+        self.running = False
+        # flag quit in main loop
+        gtk.main_quit()
+        return False
 
     def window_destroy_cb(self, window):
         """Handle destroy signal."""
-        if self.started:
-            self.shutdown()
-        self.running = False
-        gtk.main_quit()
+        self.hide()
+        glib.idle_add(self.app_destroy_handler)
     
     def map_area_expose_event_cb(self, widget, event):
         """Update desired portion of drawing area."""
@@ -380,12 +239,13 @@ class rr_announce(object):
         self.lapstart = None
         self.lapfin = None
         self.lbl_header.set_text(self.motd)
+        self.lbl_resultmsg.set_text(u'')
+        self.lbl_resultmsg.hide()
         self.final = False
         self.elap_lbl.set_text('')
         self.gap_lbl.set_text('')
         self.riders.clear()
-        self.map_redraw()		# update src map
-        self.map_area.queue_draw()	# queue copy to screen
+        self.redraw_flag = True
         
     def append_rider(self, msg):
         sr = msg.split(chr(unt4.US))
@@ -444,12 +304,13 @@ class rr_announce(object):
                         '', '', '#fefefe',None]
                 
             self.riders.append(nr)
-            self.map_redraw()		# update src map
-            self.map_area.queue_draw()	# queue copy to screen
+            self.redraw_flag = True
 
-    def msg_cb(self, m):
+    def msg_cb(self, m, nick, chan):
         """Handle message packet in main thread."""
-        redraw = False
+        if self.remoteuser and self.remoteuser.lower() != nick.lower():
+            #self.log.debug(u'Ignoring command from ' + repr(nick))
+            return False
         if m.header == 'rider':
             self.append_rider(m.text)
         #elif m.header == 'time':
@@ -468,6 +329,10 @@ class rr_announce(object):
             self.lapfin = tod.str2tod(m.text)
         elif m.header == 'resultmsg':
             self.lbl_resultmsg.set_text(m.text)
+            if m.text:
+                self.lbl_resultmsg.show()
+            else:
+                self.lbl_resultmsg.hide()
         elif m.header == u'laplbl':
             if m.text:
                 self.laplbl = m.text
@@ -497,55 +362,80 @@ class rr_announce(object):
         
     def loadconfig(self):
         """Load config from disk."""
-        cr = ConfigParser.ConfigParser({'host':USCBSRV_HOST,
-                                        'port':str(USCBSRV_PORT),
-                                        'channel':USCBSRV_CHANNEL,
-                                        'srvnick':USCBSRV_SRVNICK,
-                                        'cltnick':USCBSRV_CLTNICK,
-                                        'autoscroll':'No',
-                                        'timetick':str(TIMETICK),
-                                        'fontsize':str(FONTSIZE),
-                                        'fullscreen':'Yes',
-                                        'groupcol':'Yes',
-                                        'catcol':'Yes',
-                                        'bunchmap':'Yes',
-                                        'motd':MOTD})
-        cr.add_section('uscbsrv')
-        cr.add_section('announce')
+        cr = jsonconfig.config({u'road_announce':{
+                                    u'remoteport':u'',
+                                    u'remoteuser':u'',
+                                    u'remotechan':u'#agora',
+                                    u'autoscroll':False,
+                                    u'timetick':12,
+                                    u'fontsize':20,
+                                    u'fullscreen':False,
+                                    u'groupcol':True,
+                                    u'catcol':True,
+                                    u'bunchmap':True,
+                                    u'search':False,
+                                    u'maxlaptime':'2:00',
+                                    u'motd':MOTD}})
+        cr.add_section(u'road_announce')
+        cwfilename = metarace.default_file(CONFIGFILE)
+        # read in sysdefaults before checking for config file
+        cr.merge(metarace.sysconf, u'road_announce')
+
+        # re-set log file
+        if self.loghandler is not None:
+            self.log.removeHandler(self.loghandler)
+            self.loghandler.close()
+            self.loghandler = None
+        self.loghandler = logging.FileHandler(
+                             os.path.join(self.configpath, LOGFILE))
+        self.loghandler.setLevel(LOGHANDLER_LEVEL)
+        self.loghandler.setFormatter(logging.Formatter(
+                       '%(asctime)s %(levelname)s:%(name)s: %(message)s'))
+        self.log.addHandler(self.loghandler)
+
         # check for config file
         try:
-            a = len(cr.read(CONFIGFILE))
-            if a == 0:
-                print('No config file found - loading default values.')
-        except e:
-            print('Error reading config: ' + str(e))
+            with open(cwfilename, 'rb') as f:
+                cr.read(f)
+        except Exception as e:
+            self.log.error(u'Reading app config: ' + repr(e))
 
-        self.timetick = strops.confopt_posint(cr.get('announce', 'timetick'),
-                                              TIMETICK)
-        self.fontsize = strops.confopt_posint(cr.get('announce', 'fontsize'),
-                                              FONTSIZE)
-        self.motd = cr.get('announce', 'motd')
-        if strops.confopt_bool(cr.get('announce', 'fullscreen')):
+        self.timetick = strops.confopt_posint(cr.get(u'road_announce',
+                                              u'timetick'), TIMETICK)
+        self.fontsize = strops.confopt_posint(cr.get(u'road_announce',
+                                              u'fontsize'), FONTSIZE)
+        maxlap = tod.str2tod(cr.get(u'road_announce',u'maxlaptime'))
+        if maxlap is not None:
+            self.maxlaptime = maxlap
+        self.motd = cr.get(u'road_announce', u'motd')
+        if strops.confopt_bool(cr.get(u'road_announce', u'fullscreen')):
             self.window.fullscreen()
-        autoscroll = strops.confopt_bool(cr.get('announce', 'autoscroll'))
+        if strops.confopt_bool(cr.get(u'road_announce', u'search')):
+            self.search_ent.show()
+            self.search_ent.set_sensitive(True)
+        autoscroll = strops.confopt_bool(cr.get(u'road_announce',
+                                                u'autoscroll'))
         if autoscroll != self.autoscroll:
             self.autoscroll = True
             self.scrollcnt = 0
             glib.timeout_add(SCROLLDELAY, self.doautoscroll)
 
-        nhost = cr.get('uscbsrv', 'host')
-        nport = strops.confopt_posint(cr.get('uscbsrv', 'port'),
-                                      USCBSRV_PORT)
-        nchannel = cr.get('uscbsrv', 'channel')
-        ncltnick =cr.get('uscbsrv', 'cltnick')
-        nsrvnick = cr.get('uscbsrv', 'srvnick')
-        self.io.set_port(nhost, nport, nchannel, ncltnick, nsrvnick)
+        self.remoteport = cr.get(u'road_announce', u'remoteport')
+        self.remotechan = cr.get(u'road_announce', u'remotechan')
+        self.remoteuser = cr.get(u'road_announce', u'remoteuser')
+        self.io.set_portstr(portstr=self.remoteport,
+                             channel=self.remotechan)
+        if self.remoteuser:
+            self.log.info(u'Enabled remote control by: '
+                          + repr(self.remoteuser))
+        else:
+            self.log.info(u'Promiscuous remote control enabled.')
 
-        if not strops.confopt_bool(cr.get('announce', 'groupcol')):
+        if not strops.confopt_bool(cr.get(u'road_announce', 'groupcol')):
             self.view.get_column(5).set_visible(False)
-        if not strops.confopt_bool(cr.get('announce', 'catcol')):
+        if not strops.confopt_bool(cr.get(u'road_announce', 'catcol')):
             self.view.get_column(3).set_visible(False)
-        if not strops.confopt_bool(cr.get('announce', 'bunchmap')):
+        if not strops.confopt_bool(cr.get(u'road_announce', 'bunchmap')):
             self.map_area.hide()
 
         fnszstr = str(self.fontsize)+'px'
@@ -553,6 +443,7 @@ class rr_announce(object):
         self.elap_lbl.modify_font(pango.FontDescription(FONTFACE + ' ' + fnszstr))
         self.gap_lbl.modify_font(pango.FontDescription(FONTFACE + ' ' + fnszstr))
         self.search_entry.modify_font(pango.FontDescription(FONTFACE + ' ' + fnszstr))
+        self.lbl_resultmsg.modify_font(pango.FontDescription(FONTFACE + ' ' + fnszstr))
         self.view.modify_font(pango.FontDescription(FONTFACE + ' '+fnszstr))
 
     def doautoscroll(self):
@@ -644,11 +535,50 @@ class rr_announce(object):
             self.elap_lbl.set_text(estr)
             self.gap_lbl.set_text(dstr)
 
-    def __init__(self):
-        self.io = uscbio()
-        self.io.setcb(self.msg_cb)
+        if self.timerstat == 'running' and self.laplbl:
+            # overwrite title with lap and sprint info
+            tmsg = self.laplbl
+            if self.laptype:
+                tmsg += u' - ' + self.laptype
+            self.lbl_header.set_text(tmsg)
+
+        if self.redraw_flag:
+            self.redraw_flag = False
+            self.map_redraw()		# update src map
+            self.map_area.queue_draw()	# queue copy to screen
+
+        # check connection status
+        if not self.io.connected():
+            self.failcount += 1
+            if self.failcount > self.failthresh:
+                self.io.set_portstr(force=True)
+                self.failcount = 0
+            self.log.debug(u'Telegraph connection failed, count = '
+                           + repr(self.failcount))
+        else:
+            self.failcount = 0            
+
+    def __init__(self, configpath=None):
+        # logger and log handler
+        self.log = logging.getLogger()
+        self.log.setLevel(logging.DEBUG)
+        self.loghandler = None  # set in loadconfig to meet dir
+
+        # meet configuration path and options
+        if configpath is None:
+            configpath = u'.'   # None assumes 'current dir'
+        self.configpath = configpath
+        self.loglevel = logging.INFO    # UI log window
+        self.dosave = False
+        self.redraw_flag = False
+
+        self.io = telegraph.telegraph()
+        self.io.set_pub_cb(self.msg_cb)
+        #self.io.debug = True	# uncomment to log all IRC activity
         self.started = False
         self.running = True
+        self.failcount = 0
+        self.failthresh = 45	# connect timeout ~45sec
 
         self.autoscroll = False
         self.scrollcnt = 0
@@ -667,6 +597,7 @@ class rr_announce(object):
         self.lbl_resultmsg = b.get_object('lbl_resultmsg')
         self.lbl_resultmsg.modify_font(pango.FontDescription(FONTFACE + ' ' + fnszstr))
         self.lbl_resultmsg.set_text(u'')
+        self.search_ent =  b.get_object('search_entry')
         self.elap_lbl = b.get_object('elap_lbl')
         self.elap_lbl.set_text('-h--:--')
         self.elap_lbl.modify_font(pango.FontDescription(FONTFACE + ' '+fnszstr))
@@ -699,7 +630,7 @@ class rr_announce(object):
         t = gtk.TreeView(self.riders)
         self.view = t
         t.set_reorderable(False)
-        t.set_rules_hint(False)
+        t.set_rules_hint(True)
         t.set_headers_visible(False)
         self.search_entry = b.get_object('search_entry')
         self.search_entry.modify_font(pango.FontDescription(fnszstr))
@@ -741,13 +672,27 @@ class rr_announce(object):
 
 def main():
     """Run the announce application."""
+    configpath = None
+
+    # expand configpath on cmd line to realpath _before_ doing chdir
+    if len(sys.argv) > 2:
+        print(u'usage: road_announce [configdir]\n')
+        sys.exit(1)
+    elif len(sys.argv) == 2:
+        rdir = sys.argv[1]
+        if not os.path.isdir(rdir):
+            rdir = os.path.dirname(rdir)
+        configpath = os.path.realpath(rdir)
+
     metarace.init()
-    app = rr_announce()
+    if configpath is not None:
+        os.chdir(configpath)
+    app = rr_announce(configpath)
     app.loadconfig()
     app.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
         app.shutdown()
         raise
