@@ -1,4 +1,6 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +17,9 @@
 
 """Handler for 'sudden death' sprint rounds
 
+Winners only recorded and result is constructed as per regular sprint
+draw. For final rounds, result and seed ordering is rotated from
+normal.
 
 """
 
@@ -231,15 +236,16 @@ class sprnd(object):
             return False
         glib.idle_add(self.delayed_announce)
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
         sec = printing.sprintround()
-        sec.heading = u' '.join([
-                      u'Event', self.evno, ':',
-                        self.event[u'pref'],
-                         self.event[u'info'],
-                      u'- Start List'])
+        headvec = [u'Event', self.evno, u':', self.event[u'pref'],
+                         self.event[u'info']]
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
+
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
