@@ -20,6 +20,7 @@ from __future__ import (division, print_function, absolute_import)
 
 import re
 import os
+import random
 from . import timy	# import timy channel strings
 
 # replace codepoints 0->255 with space unless overridden
@@ -105,6 +106,10 @@ def bibstr_key(bibstr=u''):
         sval = ord(ser[0])<<12
     return sval | (bval&0xfff)
 
+def randstr():
+    """Return a string of random digits."""
+    return unicode(random.randint(10000,99999))
+
 def promptstr(prompt=u'', value=u''):
     """Prefix a non-empty string with a prompt, or return empty."""
     ret = u''
@@ -230,7 +235,7 @@ def truncpad(srcline, length, align='l', elipsis=True):
     return ret
 
 def search_name(namestr):
-    return namestr.translate(BIBLIST_UTRANS).strip().lower()
+    return namestr.translate(BIBLIST_UTRANS).strip().lower().encode('ascii','ignore')
 
 def resname_bib(bib, first, last, club):
     """Return rider name formatted for results with bib (champs/live)."""
@@ -515,7 +520,7 @@ class countback(object):
         return ret
     def __str__(self):
         ret = []
-        for i in range(1,self.maxplace()+1):
+        for i in range(0,self.maxplace()+1):
             if i in self.__store and self.__store[i] != 0:
                 ret.append(str(self.__store[i]))
             else:
