@@ -257,8 +257,9 @@ THBC_PRIORITY = glib.PRIORITY_HIGH
 THBC_UDP_PORT = 2008
 
 # Photofinish threshold - ~20cm based on tests at DISC,
-# howevever, activator period is ~20ms... more testing required!
-THBCPHOTOTHRESH = tod.tod('0.02')
+# Activator period is ~20ms... with a single active
+# loop, all reads within 20ms must be considered same time.
+THBCPHOTOTHRESH = tod.tod('0.03')
 
 # THbC protocol messages
 ESCAPE = chr(0x1b)
@@ -835,20 +836,21 @@ if __name__ == "__main__":
     t.log.addHandler(lh)
     try:
         t.start()
-        #t.setport(u'192.168.95.252')
-        t.setport(u'/dev/ttyUSB0')
+        t.setport(u'192.168.95.252')
+        #t.setport(u'/dev/ttyUSB0')
         t.sane()
-        t.setlvl(u'00', u'90')
-        t.set_date()
+        t.setlvl(u'30', u'30')
+        #t.set_date()
         t.sync()
         t.setcb(printtag)
-        #t.stop_session()
+        t.stop_session()
+        glib.timeout_add_seconds(10, showstatus, t)
         #glib.timeout_add_seconds(5, doconfig, t)
         #glib.timeout_add_seconds(4, dostop, t)
         #glib.timeout_add_seconds(6, doconfig, t)
         #glib.timeout_add_seconds(15, dostart, t)
         t.start_session()
-        gtk.main()
+        metarace.mainloop()
     except:
         t.stop_session()
         t.wait()
