@@ -548,7 +548,7 @@ if __name__ == "__main__":
             #t.printline(tmsg)
             #ucnt+=16
             #time.sleep(0.2)
-        gtk.main()
+        metarace.mainloop()
     except:
         t.exit('Exception')
         t.join()
